/*!
 *  @file		MIDI.cpp
 *  Project		MIDI Library
 *	@brief		MIDI Library for the Arduino
 *	@version	3.2
 *  @author		Francois Best 
 *	@date		24/02/11
 *  license		GPL Forty Seven Effects - 2011
 */

#include <stdint.h>
#include "MIDI.h"
#include "looptimus_mini_plus_app.h"

byte lastChannel = DEF_CHANNEL_NUM;  //Last channel a pad sequence was played on
byte lastNoteA = 128;                //Last played note A of pad sequence
byte lastNoteB = 128;                //Last played note B of pad sequence


//**TODO** - ADD 3 notes to each bank? Ignore expansion pedal?
const byte gPadA[72] = {57, 59, 61, 62, 64, 66, 58, 60, 62, 63, 65, 67, 
                       59, 61, 63, 64, 66, 68, 60, 62, 64, 65, 67, 69, 
                       61, 63, 65, 66, 68, 70, 62, 64, 66, 67, 69, 71, 
                       63, 65, 67, 68, 70, 72, 64, 66, 68, 69, 71, 73, 
                       65, 67, 69, 70, 72, 74, 66, 68, 70, 71, 73, 75, 
                       67, 69, 71, 72, 74, 76, 68, 70, 72, 73, 75, 77};

const byte gPadB[72] = {64, 66, 68, 69, 71, 73, 65, 67, 69, 70, 72, 74,
                       66, 68, 70, 71, 73, 75, 67, 69, 71, 72, 74, 76,
                       68, 70, 72, 73, 75, 77, 69, 71, 73, 74, 76, 78,
                       70, 72, 74, 75, 77, 79, 71, 73, 75, 76, 78, 80,
                       72, 74, 76, 77, 79, 81, 73, 75, 77, 78, 80, 82,
                       74, 76, 78, 79, 81, 83, 75, 77, 79, 80, 82, 84};

extern void SendMidiMessage (uint8_t *p_buf, byte len);

/*! \brief Generate and send a MIDI message from the values given.
 \param type	The message type (see type defines for reference)
 \param data1	The first data byte.
 \param data2	The second data byte (if the message contains only 1 data byte, set this one to 0).
 \param channel	The output channel on which the message will be sent (values from 1 to 16). Note: you cannot send to OMNI.
 
 This is an internal method, use it only if you need to send raw data from your code, at your own risks.
 */
void PrepareMidiMsg(kMIDIType type, byte data1, byte data2, byte channel)
{
    uint8_t message[4] = {0x90, 48, 50, 0x00};
    byte len = 0;

    // Test if channel is valid
    if ((channel >= MIDI_CHANNEL_OFF) || (channel == MIDI_CHANNEL_OMNI) || (type < NoteOff)) 
    {
        return;
    }
    if (type <= PitchBend)
    {
        // Protection: remove MSBs on data
	data1 &= 0x7F;
	data2 &= 0x7F;
        byte statusbyte = ((byte)type | ((channel-1) & 0x0F));
        message[len++] = statusbyte;
        message[len++] = data1;
        if (type != ProgramChange && type != AfterTouchChannel)
        {
            message[len++] = data2;
        }
    }
    if (type >= TuneRequest && type <= SystemReset)
    {
	// System Real-time and 1 byte.
        message[len++] = (byte)type;
    }

    SendMidiMessage (message, len);
}

/*! \brief Send a Note On message 
 \param NoteNumber	Pitch value in the MIDI format (0 to 127). Take a look at the values, names and frequencies of notes here: http://www.phys.unsw.edu.au/jw/notes.html\n
 \param Velocity	Note attack velocity (0 to 127). A NoteOn with 0 velocity is considered as a NoteOff.
 \param Channel		The channel on which the message will be sent (1 to 16). 
 */
void sendNoteOn(byte NoteNumber, byte Velocity, byte Channel)
{	
    PrepareMidiMsg(NoteOn,NoteNumber,Velocity,Channel);
}


/*! \brief Send a Note Off message (a real Note Off, not a Note On with null velocity)
 \param NoteNumber	Pitch value in the MIDI format (0 to 127). Take a look at the values, names and frequencies of notes here: http://www.phys.unsw.edu.au/jw/notes.html\n
 \param Velocity	Release velocity (0 to 127).
 \param Channel		The channel on which the message will be sent (1 to 16).
 */
void sendNoteOff(byte NoteNumber, byte Velocity, byte Channel)
{
    PrepareMidiMsg(NoteOff,NoteNumber,Velocity,Channel);
}


/*! \brief Send a Program Change message 
 \param ProgramNumber	The Program to select (0 to 127).
 \param Channel			The channel on which the message will be sent (1 to 16).
 */
void sendProgramChange(byte ProgramNumber, byte Channel)
{
    PrepareMidiMsg(ProgramChange,ProgramNumber,0,Channel);
}


/*! \brief Send a Control Change message 
 \param ControlNumber	The controller number (0 to 127). See the detailed description here: http://www.somascape.org/midi/tech/spec.html#ctrlnums
 \param ControlValue	The value for the specified controller (0 to 127).
 \param Channel			The channel on which the message will be sent (1 to 16). 
 */
void sendControlChange(byte ControlNumber, byte ControlValue, byte Channel)
{	
    PrepareMidiMsg(ControlChange,ControlNumber,ControlValue,Channel);
}

void sendPC(byte program, byte channel)
{
    sendProgramChange(program, channel);
    PrepareMidiMsg(ProgramChange,program, 0, channel);
}

#ifdef CC_MODE_SUPPORT
void sendCC(byte program, byte channel)
{
    sendControlChange(0, program, channel);
    PrepareMidiMsg(ControlChange, 0, program, channel);
}
#endif      //#ifdef CC_MODE_SUPPORT

void sendNote(byte note, byte velocity, byte channel)
{
    sendNoteOn(note, velocity, channel);  // Send a Note
    sendNoteOff(note, 0, channel);   // Stop the note
}

void stopPADLast()
{
    if(lastNoteA < 128 && lastNoteB < 128)
    {
      sendNoteOff(lastNoteA, 0, lastChannel);   // Stop the previous note A
      sendNoteOff(lastNoteB, 0, lastChannel);   // Stop the previous note B
      lastNoteA = 128;
      lastNoteB = 128;
    }
}

void sendPAD(byte data, byte velocity, byte channel)
{
    stopPADLast();
    lastNoteA = (gPadA[data] + (12 * octave));
    lastNoteB = (gPadB[data] + (12 * octave));
    sendNoteOn(lastNoteA, velocity, channel);  // Send note A
    sendNoteOn(lastNoteB, velocity, channel);  // Send note B
    lastChannel = channel;
}

void sendData(byte curMode, byte data, byte channel)
{
    if((curMode == MODE_PCC) && (data != 127))
    {
        if(data >= 128)
        {
            sendPC(pcMapA[data - 128], channel);
            sendNote(data - 4, 127, channel);
        }
        else
        {
            sendPC(pcMap[data - 1], channel);
            sendNote(data, 127, channel);
        }
    }
#ifdef CC_MODE_SUPPORT
    else if((curMode == MODE_CC) && (data != 127))
    {
        if(data >= 128)
        {
            sendCC(ccMapA[data - 128], channel);
            sendNote(data - 4, 127, channel);
        }
        else
        {
            sendCC(ccMap[data - 1], channel);
            sendNote(data, 127, channel);
        }
    }
#endif
    else if ((curMode == MODE_PAD) && (data < sizeof(gPadA)))
        sendPAD(data, 127, channel);
    else
        sendNote(data, 127, channel); 
}