/** @file
 *
 * @defgroup looptimus_plus RT9471.c
 * @{
 * @brief    configure and read data from RT9471.
 *
 * This file contains the source code for configuration and .
 *  read data from RT9471.c
 */

#include <stdint.h>
#include "boards.h"

#include "definition.h"

// Below value is in seconds
#define CHECK_BAT_CHARGE_INTERVAL   31

bool gIsExternalPowerExists = false;
static bool gsIsBatChargeDone = false;

uint8_t count_1s = 0;

extern boolean gBatLow;

uint8_t BC_ReadSTAT0Reg (void);
uint8_t BC_IsChargeDone (void);
uint8_t BC_Set_ChargingCurrent (uint8_t value);
bool IsExternalPowerExists (void);

extern void BatOperation_StartTimer (void);
extern void BatOperation_StopTimer (void);
extern void BatLowVoltage_StopTimer(void);

/*
Register 0x02: FUNCTION: 
 - no need to set anything; charing is enabled by default
*/
void RT9471_init(void)
{
    nrf_gpio_cfg_output(BAT_SCL_PIN);
    nrf_gpio_cfg_output(BAT_SDA_PIN);

    bat_i2c_init();

    Verify_RT9471 ();
    gIsExternalPowerExists = IsExternalPowerExists ();
    if (gIsExternalPowerExists)
    {
        BatLowVoltage_StopTimer();      //stop battery low voltage timer
        
        TURN_ON_EXTERNAL_POWER_INDICATION_LED
        TURN_OFF_BATTERY_POWER_INDICATION_LED
        
#if FULLCHARG_INDICATION
        gsIsBatChargeDone = BC_IsChargeDone ();
        if (false == gsIsBatChargeDone)
        {
            bsp_board_led_off(LED_DEVICE_ON_BATTERY);
        }
#endif
    }
    else
    {
        TURN_OFF_EXTERNAL_POWER_INDICATION_LED
        TURN_ON_BATTERY_POWER_INDICATION_LED
    }

    BC_EnableDisable_Watchdog (WDT_DISABLE);
    BC_Update_VOP (VAC_OVP_10pt9V);

    BC_Set_ChargingCurrent (CHARGE_CUR_350MA);
    //BC_EnableDisable_Charging (CHARGING_ENABLE);
    //BC_Update_BATFET (BATFET_DIS_IMMDTLY);

    BatOperation_StartTimer ();
    gsIsBatChargeDone = BC_IsChargeDone ();
}

void BC_StartBATFET (void)
{
    BC_Update_BATFET (BATFET_TURN_ON);
}

// This is called at every 997 miliseconds.
// 997 value is set as to avoid conflict with other timer interrupts
int g_batvolt;

void BatOperation_Timer_Handler (void *p_context)
{
    gIsExternalPowerExists = IsExternalPowerExists ();
    if (gIsExternalPowerExists)
    {
        TURN_OFF_BATTERY_POWER_INDICATION_LED

        if (0 == count_1s)
        {
            gsIsBatChargeDone = BC_IsChargeDone ();
            //if (true == gsIsBatChargeDone)
            if ((true == gsIsBatChargeDone) || (g_batvolt >= 950)) // added by nilesh parmar
            {
                TURN_ON_EXTERNAL_POWER_INDICATION_LED
            }
        }
        count_1s++;
        // Check Battery Charge status after 30 seconds
        if (count_1s >= CHECK_BAT_CHARGE_INTERVAL)
        {
            count_1s = 0;
        }
        // Toggle Green LED at every 1 second
        if (true != gsIsBatChargeDone)
            TOGGLE_EXTERNAL_POWER_INDICATION_LED
    }
    else
    {
        TURN_OFF_EXTERNAL_POWER_INDICATION_LED
        
        if(!gBatLow)
        {
           TURN_ON_BATTERY_POWER_INDICATION_LED
        }
    }
}

byte gBatDeviceID = 0;

static uint8_t BC_Read_DeviceID (void)
{
    uint8_t deviceID = 0;

    deviceID = bat_i2c_receive_byte_data (BC_IC_I2C_ADD, REG_INFO_ADD);

#if RESET_REG
    deviceID |= 0x80;       // Reset Register

    bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_INFO_ADD, deviceID);
#endif

    deviceID &= DEVICE_ID_MASK;

    gBatDeviceID = (deviceID >> DEVICE_ID_POS);

    return gBatDeviceID;
}

static uint8_t BC_Read_Register (uint8_t regVal)
{
    uint8_t readVal = 0;

    switch (regVal)
    {
        default:
            return 0;

        case REG_TOP_ADD:
        case REG_FUN_ADD:
        case REG_IBUS_ADD:
        case REG_VBUS_ADD:
        case REG_PRECHG_ADD:
        case REG_VCHG_ADD:
        case REG_ICHG_ADD:
        case REG_INFO_ADD:
        case REG_ICSTAT_ADD:
        case REG_STAT0_ADD:
            readVal = bat_i2c_receive_byte_data (BC_IC_I2C_ADD, regVal);
            break;
    }

    return readVal;
}

uint8_t Verify_RT9471 (void)
{
    uint8_t deviceID = 0;

    deviceID = BC_Read_DeviceID ();

    if (DEVICE_ID_RT9471 == deviceID)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

uint8_t BC_Set_ChargingCurrent (uint8_t value)
{
    uint8_t funcRegVal = 0;

    if (value <= CHARGE_CUR_350MA)
    {
        return (bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_ICHG_ADD, value));
    }

    return 0;
}

uint8_t BC_EnableDisable_Charging (uint8_t enable)
{
    uint8_t funcRegVal = 0;

    funcRegVal = BC_Read_Register(REG_FUN_ADD);

    enable ? (funcRegVal |= 0x01) : (funcRegVal &= 0xFE);
    
    return (bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_FUN_ADD, funcRegVal));
}


uint8_t BC_Update_VOP (uint8_t value)
{
    uint8_t funcRegVal = 0;

    funcRegVal = BC_Read_Register(REG_VBUS_ADD);

    funcRegVal |= ((value << VAC_OVP_POS) & VAC_OVP_MASK);   // set '0' bit 7 and 6;

    bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_VBUS_ADD, funcRegVal);

    //-------------------------------------------
#if 0
    funcRegVal = BC_Read_Register(REG_IBUS_ADD);

    funcRegVal |= 0x80;     // Enable AICC function

    //bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_IBUS_ADD, funcRegVal);

    //-------------------------------------------
    funcRegVal = BC_Read_Register(REG_PRECHG_ADD);
    
    funcRegVal &= 0xF0;
    funcRegVal |= 0x00;     // Set 50mA Pre-charge current
    
    bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_PRECHG_ADD, funcRegVal);

    //-------------------------------------------
#endif        
    return 1;
}

byte gIcStatus = 0;

#pragma -O1
uint8_t funcRegVal = 0;

uint8_t BC_IsChargeDone (void)
{
    uint8_t funcRegVal = 0;

    funcRegVal = BC_Read_Register(REG_ICSTAT_ADD);

    funcRegVal &= IC_STAT_MASK;   // set '0' bit 7 and 6;

    gIcStatus = funcRegVal;

    if (gIcStatus == IC_STAT_CHARGE_DONE)
    {
        return 1;
    }
    else
    {
        return 0;
    }

    return 1;
}


byte gStat0Reg = 0;

#pragma -O1

uint8_t BC_ReadSTAT0Reg (void)
{
    gStat0Reg = BC_Read_Register(REG_STAT0_ADD);

    return 1;
}

bool IsExternalPowerExists (void)
{
   BC_ReadSTAT0Reg();
   return ((gStat0Reg >> ST_VBUS_GD_POS) & ST_VBUS_GD_MASK);
}

uint8_t BC_Update_BATFET (uint8_t value)
{
    uint8_t funcRegVal = 0, temp = 0;

    funcRegVal = BC_Read_Register(REG_FUN_ADD);

    temp = funcRegVal;

    funcRegVal &= (~(BATFET_DIS_DLY_MASK));   // set '0' bit 7 and 6;

    value = (value << BATFET_DIS_DLY_POS);

    funcRegVal |= value;

    return (bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_FUN_ADD, funcRegVal));

    BC_Update_VOP(VAC_OVP_10pt9V);

    return (bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_FUN_ADD, 0x01));
}

uint8_t BC_EnableDisable_Watchdog (uint8_t value)
{
    uint8_t topRegVal = 0;

    topRegVal = BC_Read_Register(REG_TOP_ADD);

    topRegVal &= (~(WDT_MASK));   // set '0' bit 7 and 6;

    value = (value << WDT_POS);

    topRegVal |= value;
    
    return (bat_i2c_send_byte_data (BC_IC_I2C_ADD, REG_TOP_ADD, topRegVal));
}