/**
 * Copyright (c) 2014 - 2020, Nordic Semiconductor ASA
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form, except as embedded into a Nordic
 *    Semiconductor ASA integrated circuit in a product or a software update for
 *    such product, must reproduce the above copyright notice, this list of
 *    conditions and the following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. Neither the name of Nordic Semiconductor ASA nor the names of its
 *    contributors may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * 4. This software, with or without modification, must only be used with a
 *    Nordic Semiconductor ASA integrated circuit.
 *
 * 5. Any software provided in binary form under this license must not be reverse
 *    engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY NORDIC SEMICONDUCTOR ASA "AS IS" AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL NORDIC SEMICONDUCTOR ASA OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/** @file
 * @defgroup flashwrite_example_main main.c
 * @{
 * @ingroup flashwrite_example
 *
 * @brief This file contains the source code for a sample application using the Flash Write Application.
 *a
 */

#include <stdbool.h>
#include <stdio.h>
#include "bsp.h"
#ifdef BLE_MIDI_SUPPORT
#include "nrf_fstorage_sd.h"
#endif
#include "nrf_nvmc.h"
#include "FreeRTOS.h"
#include "task.h"


#include "nrf_delay.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"

#include "looptimus_config.h"

//#include "definition.h"

#define FLASHWRITE_EXAMPLE_BLOCK_VALID          (0xA55A5AA5)
#define FLASHWRITE_EXAMPLE_BLOCK_INVALID        (0xA55A0000)
#define FLASHWRITE_EXAMPLE_BLOCK_NOT_INIT       (0xFFFFFFFF)
#define CONFIG_DATA_FLASH_PAGE_NUMBER           0x6F
#define CONFIG_DATA_FLASH_PAGE_SIZE             0x1000

bool gWritePageAfterErase = false;

#ifdef BLE_MIDI_SUPPORT
static void fstorage_evt_handler(nrf_fstorage_evt_t * p_evt);

NRF_FSTORAGE_DEF(nrf_fstorage_t fstorage) =
{
    /* Set a handler for fstorage events. */
    .evt_handler = fstorage_evt_handler,

    /* These below are the boundaries of the flash space assigned to this instance of fstorage.
     * You must set these manually, even at runtime, before nrf_fstorage_init() is called.
     * The function nrf5_flash_end_addr_get() can be used to retrieve the last address on the
     * last page of flash available to write data. */
    .start_addr = (CONFIG_DATA_FLASH_PAGE_NUMBER * CONFIG_DATA_FLASH_PAGE_SIZE),
    .end_addr   = ((CONFIG_DATA_FLASH_PAGE_NUMBER * CONFIG_DATA_FLASH_PAGE_SIZE) + CONFIG_DATA_FLASH_PAGE_SIZE),
};

nrf_mutex_t flash_write_mutex;
#endif      //#ifdef BLE_MIDI_SUPPORT

typedef struct
{
   uint32_t magic_number;
   looptimus_config_data_t config_data;
} struct_flash_data_t;

typedef struct
{
    uint32_t addr;
    uint32_t pg_size;
    uint32_t pg_num;
    struct_flash_data_t * m_p_flash_data;
} struct_looptimus_flash_data_t;

static struct_looptimus_flash_data_t m_data;

struct_flash_data_t * p_RdConfigData = NULL;

#ifdef BLE_MIDI_SUPPORT
static uint32_t invalid_data_header = 0, valid_data_header = 0;
#endif

static void flashwrite_read_cmd(looptimus_config_data_t *pConfigData);
static void write_config_data(looptimus_config_data_t *pConfigData);

#ifndef BLE_MIDI_SUPPORT
extern void PartialPageErase_StartTimer (void);
#endif  //#ifndef BLE_MIDI_SUPPORT
extern void FlashWrite_StartTimer (void);

#ifdef BLE_MIDI_SUPPORT
static void fstorage_evt_handler(nrf_fstorage_evt_t * p_evt)
{
    if (p_evt->result != NRF_SUCCESS)
    {
        NRF_LOG_INFO("--> Event received: ERROR while executing an fstorage operation.");
        return;
    }

    switch (p_evt->id)
    {
        case NRF_FSTORAGE_EVT_WRITE_RESULT:
        {
            NRF_LOG_INFO("--> Event received: wrote %d bytes at address 0x%x.",
                         p_evt->len, p_evt->addr);
        } break;

        case NRF_FSTORAGE_EVT_ERASE_RESULT:
        {
            NRF_LOG_INFO("--> Event received: erased %d page from address 0x%x.",
                         p_evt->len, p_evt->addr);
            if (gWritePageAfterErase)
            {
                gWritePageAfterErase = false;
                gUpdateFlashData = true;
            }
        } break;

        default:
            break;
    }
}

#endif      //#ifdef BLE_MIDI_SUPPORT

void Config_Memory_init(void)
{

    // To Erase config page
    //nrf_nvmc_page_erase((CONFIG_DATA_FLASH_PAGE_NUMBER * CONFIG_DATA_FLASH_PAGE_SIZE));

    m_data.pg_num = CONFIG_DATA_FLASH_PAGE_NUMBER;   //NRF_FICR->CODESIZE - 1;
    m_data.pg_size = NRF_FICR->CODEPAGESIZE;
    m_data.addr = (m_data.pg_num * m_data.pg_size);

    m_data.m_p_flash_data = (struct_flash_data_t *)m_data.addr;

#ifdef BLE_MIDI_SUPPORT
    ret_code_t rc;
    
    nrf_fstorage_api_t * p_fs_api;
    p_fs_api = &nrf_fstorage_sd;

    rc = nrf_fstorage_init(&fstorage, p_fs_api, NULL);
    APP_ERROR_CHECK(rc);

    sd_mutex_new (&flash_write_mutex);
#endif
    


#if 0//def BLE_MIDI_SUPPORT
    while (1)
    {
        if (m_data.m_p_flash_data->magic_number == FLASHWRITE_EXAMPLE_BLOCK_NOT_INIT)
        {
            if ((uint32_t)m_data.m_p_flash_data != (fstorage.start_addr))
            {
                --m_data.m_p_flash_data;
            }
            break;
        }

        ++m_data.m_p_flash_data;
        if ((uint32_t)m_data.m_p_flash_data >= fstorage.end_addr)
        {
            nrf_nvmc_page_erase(fstorage.start_addr);
            m_data.m_p_flash_data = (struct_flash_data_t *)fstorage.start_addr;
            break;
        }
    }
#else
    while (1)
    {
        if (m_data.m_p_flash_data->magic_number == FLASHWRITE_EXAMPLE_BLOCK_VALID)
        {
            p_RdConfigData = m_data.m_p_flash_data;
            break;
        }

        if (m_data.m_p_flash_data->magic_number == FLASHWRITE_EXAMPLE_BLOCK_INVALID)
        {            
            ++m_data.m_p_flash_data;
            continue;
        }
        nrf_nvmc_page_erase(m_data.addr);
        nrf_delay_ms(100);
        m_data.m_p_flash_data = (struct_flash_data_t *)m_data.addr;
        break;
    }
#endif
    
    p_RdConfigData = m_data.m_p_flash_data;
}

static void flashwrite_erase_cmd(void)
{
    byte ret = 3;
#ifndef BLE_MIDI_SUPPORT
    nrf_nvmc_page_erase(m_data.addr);
#else
    ret = nrf_fstorage_erase(&fstorage, m_data.addr, 1, NULL);
    if (ret != NRF_SUCCESS)
    { /* TODO */ }    
#endif  //#ifndef BLE_MIDI_SUPPORT

    m_data.m_p_flash_data = (struct_flash_data_t *)m_data.addr;
}

#ifndef BLE_MIDI_SUPPORT
void Flash_PartialPage_Erase (void)
{
    nrf_nvmc_page_partial_erase (m_data.addr); // Do partial page erase
}
#endif

static void flashwrite_read_cmd (looptimus_config_data_t *pConfigData)
{
    struct_flash_data_t * p_data = m_data.m_p_flash_data;

    if (p_data->magic_number != FLASHWRITE_EXAMPLE_BLOCK_VALID)
    {
        NRF_LOG_RAW_INFO("Please write something first");
        return;
    }

    //while (p_data <= m_data.m_p_flash_data)
    {
        if ((p_data->magic_number != FLASHWRITE_EXAMPLE_BLOCK_VALID) &&
            (p_data->magic_number != FLASHWRITE_EXAMPLE_BLOCK_INVALID))
        {
            NRF_LOG_RAW_INFO("Corrupted data found");
            return;
        }
        *pConfigData = p_data->config_data;
        p_RdConfigData = p_data;
    }
}

uint32_t gDiff = 0;
looptimus_config_data_t tempWriteConfig;
static void write_config_data(looptimus_config_data_t *pConfigData)
{
    static uint16_t const page_size = 4096;

    if ((m_data.m_p_flash_data->magic_number != FLASHWRITE_EXAMPLE_BLOCK_NOT_INIT) &&
        (m_data.m_p_flash_data->magic_number != FLASHWRITE_EXAMPLE_BLOCK_VALID))
    {
        NRF_LOG_RAW_INFO("Flash corrupted, please errase it first.");
        return;
    }

    if (m_data.m_p_flash_data->magic_number == FLASHWRITE_EXAMPLE_BLOCK_VALID)
    {
        uint32_t new_end_addr = (uint32_t)(m_data.m_p_flash_data) + (2 * sizeof(looptimus_config_data_t));
        uint16_t diff = (new_end_addr - m_data.addr);
        gDiff = diff;
        if (diff > page_size)
        //if (diff > (sizeof(looptimus_config_data_t) * 3))
        {
            NRF_LOG_RAW_INFO("Not enough space - please erase flash first: %d %d", new_end_addr, diff);
            gEraseConfigData_f = true;
            gWritePageAfterErase = true;
            gUpdateFlashData = false;
            return;
        }
#ifndef BLE_MIDI_SUPPORT
        nrf_nvmc_write_word((uint32_t)&m_data.m_p_flash_data->magic_number,
                            FLASHWRITE_EXAMPLE_BLOCK_INVALID);
#else
        invalid_data_header = FLASHWRITE_EXAMPLE_BLOCK_INVALID;
        nrf_fstorage_write (&fstorage, (uint32_t)&m_data.m_p_flash_data->magic_number, 
                            &invalid_data_header, sizeof(invalid_data_header), NULL);
        
#endif      //#ifndef BLE_MIDI_SUPPORT

        ++m_data.m_p_flash_data;
    }

    p_RdConfigData = m_data.m_p_flash_data;

    // Write Configuration data
#ifndef BLE_MIDI_SUPPORT
    nrf_nvmc_write_words((uint32_t)&m_data.m_p_flash_data->config_data,
                        (uint32_t *)pConfigData, sizeof(looptimus_config_data_t)/4);
    // Update magic number
    nrf_nvmc_write_word((uint32_t)&m_data.m_p_flash_data->magic_number,
                        FLASHWRITE_EXAMPLE_BLOCK_VALID);

#else
    
    sd_mutex_acquire (&flash_write_mutex);
    tempWriteConfig = *pConfigData;
    
    // Write Configuration data
    nrf_fstorage_write (&fstorage, (uint32_t)&m_data.m_p_flash_data->config_data, 
                         (uint32_t *)&tempWriteConfig, sizeof(looptimus_config_data_t), NULL);
    
    // Update magic number
    valid_data_header = FLASHWRITE_EXAMPLE_BLOCK_VALID;
    nrf_fstorage_write (&fstorage, (uint32_t)&m_data.m_p_flash_data->magic_number,
                        &valid_data_header, sizeof(valid_data_header), NULL);

    sd_mutex_release (&flash_write_mutex);

#endif      //#ifndef BLE_MIDI_SUPPORT
}

/** @} */

void ReadConfigData (void)
{
    flashwrite_read_cmd (&gLooptimusConfigData);
}

void WriteConfigData (void)
{
    looptimus_config_data_t *pConfigData = &gLooptimusConfigData;

    write_config_data (pConfigData);
}

void EraseConfigData (void)
{
    flashwrite_erase_cmd();
}

byte Read8Bits_FromMemory (int offset, int pos)
{
    byte temp = 0;

    temp = ( ( *( ((uint32_t*)p_RdConfigData) + offset) >> (pos * 8))) & 0xFF;

    return temp;
}

int Read8Bits_Int_FromMemory (int offset, int pos)
{
    byte temp = 0;

    temp = ( ( *( ((uint32_t*)p_RdConfigData)) >> (pos * 8))) & 0xFF;

    return temp;
}

unsigned short int Read16Bits_FromMemory (int offset, int pos)
{
    unsigned short int temp = 0;

    temp = ( ( *( ((uint32_t*)p_RdConfigData) + offset) >> (pos * 8))) & 0xFFFF;

    return temp;
}

unsigned int Read32Bits_FromMemory (int offset)
{
    unsigned int temp = 0;

    temp =  ( *( ((uint32_t*)p_RdConfigData) + offset) );

    return temp;
}

void mem_operation_thread (void *arg)
{
    UNUSED_PARAMETER(arg);

    while (true)
    {
        if (gUpdateFlashData == true)
        {
            FlashWrite_StartTimer();
            gUpdateFlashData = false;
        }
        if (gEraseConfigData_f == true)     // When a Page is to erase
        {
        #ifndef BLE_MIDI_SUPPORT
            PartialPageErase_StartTimer ();
        #else
            EraseConfigData ();
            gEraseConfigData_f = false;
        #endif
        }
        //nrf_delay_ms(50);
        vTaskSuspend(NULL); // Suspend myself
    }
}